@@ -1,7 +1,8 @@
 // -*- C++ -*-
 //
 // This file is part of HepMC
-// Copyright (C) 2014-2020 The HepMC collaboration (see AUTHORS for details)
+// Copyright (C) 2014-2023 The HepMC collaboration (see AUTHORS for details)
+//
 /**
  *  @file Errors.h
  *  @brief Implementation of  error and HEPMC3_HEPMC3_WARNING macros
@@ -12,12 +13,12 @@
 
 #include <iostream>
 #include <stdexcept>
-
+#include <HepMC3/Setup.h>
 namespace HepMC3 {
 
 
 /// @name Printing macros
-//@{
+/// @{
 
 /** @brief Macro for printing error messages */
 #define HEPMC3_ERROR(MESSAGE)   if ( Setup::print_errors() )   { std::cerr << "ERROR::" << MESSAGE << std::endl; }
@@ -29,7 +30,7 @@ namespace HepMC3 {
 #ifndef HEPMC3_RELEASE_VERSION
 
 /** @brief Macro for printing debug messages with appropriate debug level */
-#define HEPMC3_DEBUG(LEVEL,MESSAGE) if( Setup::debug_level()>=(LEVEL) ) { std::cout << "DEBUG(" << LEVEL <<")::" << MESSAGE << std::endl; }
+#define HEPMC3_DEBUG(LEVEL,MESSAGE) if ( Setup::debug_level()>=(LEVEL) ) { std::cout << "DEBUG(" << LEVEL <<")::" << MESSAGE << std::endl; }
 /** @brief Macro for storing code useful for debugging */
 #define HEPMC3_DEBUG_CODE_BLOCK( x ) x
 
@@ -40,10 +41,7 @@ namespace HepMC3 {
 
 #endif
 
-//@}
-
-
-
+/// @}
 
 } // namespace HepMC3
 
