@@ -1,29 +1,29 @@
 // -*- C++ -*-
 //
 // This file is part of HepMC
-// Copyright (C) 2014-2020 The HepMC collaboration (see AUTHORS for details)
+// Copyright (C) 2014-2023 The HepMC collaboration (see AUTHORS for details)
 //
 /**
  *  @file ReaderAsciiHepMC2.cc
  *  @brief Implementation of \b class ReaderAsciiHepMC2
  *
  */
-#include <cstring>
+#include <array>
 #include <cstdlib>
-
-#include "HepMC3/ReaderAsciiHepMC2.h"
+#include <cstring>
 
 #include "HepMC3/GenEvent.h"
-#include "HepMC3/GenVertex.h"
-#include "HepMC3/GenParticle.h"
 #include "HepMC3/GenHeavyIon.h"
+#include "HepMC3/GenParticle.h"
 #include "HepMC3/GenPdfInfo.h"
+#include "HepMC3/GenVertex.h"
+#include "HepMC3/ReaderAsciiHepMC2.h"
 #include "HepMC3/Setup.h"
 
 namespace HepMC3 {
 
 ReaderAsciiHepMC2::ReaderAsciiHepMC2(const std::string& filename):
-    m_file(filename), m_stream(0), m_isstream(false) {
+    m_file(filename), m_stream(nullptr), m_isstream(false) {
     if ( !m_file.is_open() ) {
         HEPMC3_ERROR("ReaderAsciiHepMC2: could not open input file: " << filename )
     }
@@ -41,20 +41,29 @@ ReaderAsciiHepMC2::ReaderAsciiHepMC2(std::istream & stream)
     m_event_ghost = new GenEvent();
 }
 
+ReaderAsciiHepMC2::ReaderAsciiHepMC2(std::shared_ptr<std::istream> s_stream)
+    : m_shared_stream(s_stream), m_stream(s_stream.get()), m_isstream(true)
+{
+    if ( !m_stream->good() ) {
+        HEPMC3_ERROR("ReaderAsciiHepMC2: could not open input stream ")
+    }
+    set_run_info(std::make_shared<GenRunInfo>());
+}
+
+
 ReaderAsciiHepMC2::~ReaderAsciiHepMC2() { if (m_event_ghost) { m_event_ghost->clear(); delete m_event_ghost; m_event_ghost=nullptr; } if (!m_isstream) close(); }
 
 bool ReaderAsciiHepMC2::skip(const int n)
 {
-    const size_t       max_buffer_size = 512*512;
-    char               buf[max_buffer_size];
+    std::array<char, 262144> buf;
     int nn = n;
     while (!failed()) {
-        char peek;
+        char peek(0);
         if ( (!m_file.is_open()) && (!m_isstream) ) return false;
         m_isstream ? peek = m_stream->peek() : peek = m_file.peek();
         if ( peek =='E' ) nn--;
-        if (nn < 0) return true;
-        m_isstream ? m_stream->getline(buf, max_buffer_size) : m_file.getline(buf, max_buffer_size);
+        if (nn < 0) { return true; }
+        m_isstream ? m_stream->getline(buf.data(), buf.size()) : m_file.getline(buf.data(), buf.size());
     }
     return true;
 }
@@ -62,9 +71,8 @@ bool ReaderAsciiHepMC2::skip(const int n)
 bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
     if ( (!m_file.is_open()) && (!m_isstream) ) return false;
 
-    char               peek;
-    const size_t  max_buffer_size = 512*512;
-    char          buf[max_buffer_size];
+    char               peek = 0;
+    std::array<char, 262144> buf;
     bool          parsed_event_header            = false;
     bool          is_parsing_successful          = true;
     int           parsing_result                 = 0;
@@ -87,14 +95,14 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
     // Parse event, vertex and particle information
     //
     while (!failed()) {
-        m_isstream ? m_stream->getline(buf, max_buffer_size) : m_file.getline(buf, max_buffer_size);
-        if ( strlen(buf) == 0 ) continue;
+        m_isstream ? m_stream->getline(buf.data(), buf.size()) : m_file.getline(buf.data(), buf.size());
+        if ( strlen(buf.data()) == 0 ) continue;
         // Check for IO_GenEvent header/footer
-        if ( strncmp(buf, "HepMC", 5) == 0 ) {
-            if ( strncmp(buf, "HepMC::Version", 14) != 0 && strncmp(buf, "HepMC::IO_GenEvent", 18) != 0 )
+        if ( strncmp(buf.data(), "HepMC", 5) == 0 ) {
+            if ( strncmp(buf.data(), "HepMC::Version", 14) != 0 && strncmp(buf.data(), "HepMC::IO_GenEvent", 18) != 0 )
             {
                 HEPMC3_WARNING("ReaderAsciiHepMC2: found unsupported expression in header. Will close the input.")
-                std::cout <<buf << std::endl;
+                std::cout <<buf.data() << std::endl;
                 m_isstream ? m_stream->clear(std::ios::eofbit) : m_file.clear(std::ios::eofbit);
             }
             if (parsed_event_header) {
@@ -105,7 +113,7 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
         }
         switch (buf[0]) {
         case 'E':
-            parsing_result = parse_event_information(evt, buf);
+            parsing_result = parse_event_information(evt, buf.data());
             if (parsing_result < 0) {
                 is_parsing_successful = false;
                 HEPMC3_ERROR("ReaderAsciiHepMC2: HEPMC3_ERROR parsing event information")
@@ -118,7 +126,8 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
                 m_particle_cache_ghost.reserve(vertices_count*3);
                 m_vertex_barcodes.reserve(vertices_count);
                 m_end_vertex_barcodes.reserve(vertices_count*3);
-                evt.reserve(vertices_count,vertices_count*3);
+                // Here we make a trick: reserve for this event the vertices_count*3 or the number of particles in the prev. event.
+                evt.reserve(vertices_count, m_particle_cache_ghost.capacity());
                 is_parsing_successful = true;
             }
             parsed_event_header = true;
@@ -134,7 +143,7 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
             }
             current_vertex_particles_parsed = 0;
 
-            parsing_result = parse_vertex_information(buf);
+            parsing_result = parse_vertex_information(buf.data());
 
             if (parsing_result < 0) {
                 is_parsing_successful = false;
@@ -147,7 +156,7 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
             break;
         case 'P':
 
-            parsing_result   = parse_particle_information(buf);
+            parsing_result   = parse_particle_information(buf.data());
 
             if (parsing_result < 0) {
                 is_parsing_successful = false;
@@ -159,19 +168,19 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
             }
             break;
         case 'U':
-            is_parsing_successful = parse_units(evt, buf);
+            is_parsing_successful = parse_units(evt, buf.data());
             break;
         case 'F':
-            is_parsing_successful = parse_pdf_info(evt, buf);
+            is_parsing_successful = parse_pdf_info(evt, buf.data());
             break;
         case 'H':
-            is_parsing_successful = parse_heavy_ion(evt, buf);
+            is_parsing_successful = parse_heavy_ion(evt, buf.data());
             break;
         case 'N':
-            is_parsing_successful = parse_weight_names(buf);
+            is_parsing_successful = parse_weight_names(buf.data());
             break;
         case 'C':
-            is_parsing_successful = parse_xs_info(evt, buf);
+            is_parsing_successful = parse_xs_info(evt, buf.data());
             break;
         default:
             HEPMC3_WARNING("ReaderAsciiHepMC2: skipping unrecognised prefix: " << buf[0])
@@ -201,10 +210,17 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
 
     if ( !is_parsing_successful ) {
         HEPMC3_ERROR("ReaderAsciiHepMC2: event parsing failed. Returning empty event")
-        HEPMC3_DEBUG(1, "Parsing failed at line:" << std::endl << buf)
+        HEPMC3_DEBUG(1, "Parsing failed at line:" << std::endl << buf.data())
         evt.clear();
         m_isstream ? m_stream->clear(std::ios::badbit) : m_file.clear(std::ios::badbit);
-        return 0;
+        return false;
+    }
+    if (run_info() && run_info()->weight_names().empty() ) {
+        run_info()->set_weight_names(std::vector<std::string> {"Default"});
+    }
+    if (evt.weights().empty()) {
+        HEPMC3_WARNING("ReaderAsciiHepMC2: weights are empty, an event weight 1.0 will be added.")
+        evt.weights().push_back(1.0);
     }
 
     // Restore production vertex pointers
@@ -221,22 +237,26 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
 
     // Remove vertices with no incoming particles or no outgoing particles
     for (unsigned int i = 0; i < m_vertex_cache.size(); ++i) {
-        if ( m_vertex_cache[i]->particles_in().size() == 0 ) {
-            HEPMC3_DEBUG(30, "ReaderAsciiHepMC2::read_event - found a vertex without incoming particles: " << m_vertex_cache[i]->id());
+        if ( m_vertex_cache[i]->particles_in().empty() ) {
+            HEPMC3_DEBUG(30, "ReaderAsciiHepMC2::read_event - found a vertex without incoming particles: " << m_vertex_cache[i]->id() );
             //Sometimes the root vertex has no incoming particles.  Here we try to save the event.
             std::vector<GenParticlePtr> beams;
-            for (auto p: m_vertex_cache[i]->particles_out()) if (p->status() == 4 && !(p->end_vertex())) beams.push_back(p);
-            for (auto p: beams)
+            beams.reserve(2);
+            for (const auto& p: m_vertex_cache[i]->particles_out()) if (p->status() == 4 && !(p->end_vertex())) beams.emplace_back(p);
+            for (auto& p: beams)
             {
                 m_vertex_cache[i]->add_particle_in(p);
                 m_vertex_cache[i]->remove_particle_out(p);
                 HEPMC3_DEBUG(30, "ReaderAsciiHepMC2::read_event - moved particle with status=4 from the outgoing to the incoming particles of vertex: " << m_vertex_cache[i]->id());
             }
-            if (beams.size() == 0) m_vertex_cache[i] = nullptr;
+            if (beams.empty()) {
+                HEPMC3_DEBUG(30, "ReaderAsciiHepMC2::read_event - removed vertex without incoming particles: " << m_vertex_cache[i]->id() );
+                m_vertex_cache[i] = nullptr;
+            }
         }
-        else if ( m_vertex_cache[i]->particles_out().size() == 0 ) {
-            HEPMC3_DEBUG(30, "ReaderAsciiHepMC2::read_event - found a vertex without outgoing particles: " << m_vertex_cache[i]->id());
+        else if ( m_vertex_cache[i]->particles_out().empty() ) {
             m_vertex_cache[i] = nullptr;
+            HEPMC3_DEBUG(30, "ReaderAsciiHepMC2::read_event - removed vertex without outgoing particles: " << m_vertex_cache[i]->id());
         }
     }
 
@@ -246,54 +266,59 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
     // Add whole event tree in topological order
     evt.add_tree(m_particle_cache);
 
-    if (m_options.find("event_random_states_are_separated") != m_options.end())
+    if (m_options.count("event_random_states_are_separated") != 0)
     {
         std::shared_ptr<VectorLongIntAttribute> random_states_a = evt.attribute<VectorLongIntAttribute>("random_states");
         if (random_states_a) {
             std::vector<long int> random_states_v = random_states_a->value();
-            for (size_t i = 0; i < random_states_v.size(); ++i )
+            for (size_t i = 0; i < random_states_v.size(); ++i ) {
                 evt.add_attribute("random_states" + std::to_string((long long unsigned int)i), std::make_shared<IntAttribute>(random_states_v[i]));
+            }
             evt.remove_attribute("random_states");
         }
 
     }
 
     std::map< std::string, std::map<int, std::shared_ptr<Attribute> > > cached_attributes = m_event_ghost->attributes();
-    if (cached_attributes.find("flows") != cached_attributes.end()) {
-        std::map<int, std::shared_ptr<Attribute> > flows = cached_attributes.at("flows");
-        if (m_options.find("particle_flows_are_separated") == m_options.end()) {
-            for (auto f: flows) if (f.first > 0 && f.first <= (int)m_particle_cache.size())  m_particle_cache[f.first-1]->add_attribute("flows", f.second);
+    if (cached_attributes.count("flows") != 0) {
+        const std::map<int, std::shared_ptr<Attribute> >& flows = cached_attributes.at("flows");
+        if (m_options.count("particle_flows_are_separated") == 0) {
+            for (const auto& f: flows) if (f.first > 0 && f.first <= (int)m_particle_cache.size()) {  m_particle_cache[f.first-1]->add_attribute("flows", f.second);}
         } else  {
-            for (auto f: flows) if (f.first > 0 && f.first <= (int)m_particle_cache.size()) {
+            for (const auto& f: flows) {
+                if (f.first > 0 && f.first <= (int)m_particle_cache.size()) {
                     std::shared_ptr<VectorIntAttribute>  casted = std::dynamic_pointer_cast<VectorIntAttribute>(f.second);
                     if (!casted) continue;//Should not happen
                     std::vector<int> this_p_flow = casted->value();
                     for (size_t i = 0; i<this_p_flow.size(); i++) m_particle_cache[f.first-1]->add_attribute("flow" + std::to_string(i + 1), std::make_shared<IntAttribute>(this_p_flow[i]));
                 }
+            }
         }
     }
 
-    if (cached_attributes.find("phi") != cached_attributes.end()) {
-        std::map<int, std::shared_ptr<Attribute> > phi = cached_attributes.at("phi");
-        for (auto f: phi) if (f.first > 0 &&f.first <= (int)m_particle_cache.size())  m_particle_cache[f.first-1]->add_attribute("phi", f.second);
+    if (cached_attributes.count("phi") != 0) {
+        const std::map<int, std::shared_ptr<Attribute> >& phi = cached_attributes.at("phi");
+        for (const auto& f: phi) if (f.first > 0 &&f.first <= (int)m_particle_cache.size())  m_particle_cache[f.first-1]->add_attribute("phi", f.second);
     }
 
-    if (cached_attributes.find("theta") != cached_attributes.end()) {
-        std::map<int, std::shared_ptr<Attribute> > theta = cached_attributes.at("theta");
-        for (auto f: theta) if (f.first > 0 && f.first <= (int)m_particle_cache.size())  m_particle_cache[f.first-1]->add_attribute("theta", f.second);
+    if (cached_attributes.count("theta") != 0) {
+        const std::map<int, std::shared_ptr<Attribute> >& theta = cached_attributes.at("theta");
+        for (const auto& f: theta) if (f.first > 0 && f.first <= (int)m_particle_cache.size())  m_particle_cache[f.first-1]->add_attribute("theta", f.second);
     }
 
-    if (cached_attributes.find("weights") != cached_attributes.end()) {
-        std::map<int, std::shared_ptr<Attribute> > weights = cached_attributes.at("weights");
-        if (m_options.find("vertex_weights_are_separated") == m_options.end()) {
-            for (auto f: weights) if (f.first < 0 && f.first >= -(int)m_vertex_cache.size())  m_vertex_cache[-f.first-1]->add_attribute("weights", f.second);
+    if (cached_attributes.count("weights") != 0) {
+        const std::map<int, std::shared_ptr<Attribute> >& weights = cached_attributes.at("weights");
+        if (m_options.count("vertex_weights_are_separated") == 0) {
+            for (const auto& f: weights) { if (f.first < 0 && f.first >= -(int)m_vertex_cache.size())  m_vertex_cache[-f.first-1]->add_attribute("weights", f.second);}
         } else {
-            for (auto f: weights) if (f.first < 0 && f.first >= -(int)m_vertex_cache.size()) {
+            for (const auto& f: weights) {
+                if (f.first < 0 && f.first >= -(int)m_vertex_cache.size()) {
                     std::shared_ptr<VectorDoubleAttribute>  casted = std::dynamic_pointer_cast<VectorDoubleAttribute>(f.second);
                     if (!casted) continue;//Should not happen
                     std::vector<double> this_v_weight = casted->value();
                     for (size_t i = 0; i < this_v_weight.size(); i++) m_particle_cache[-f.first-1]->add_attribute("weight"+std::to_string(i), std::make_shared<DoubleAttribute>(this_v_weight[i]));
                 }
+            }
         }
     }
 
@@ -312,12 +337,12 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
     m_particle_cache_ghost.clear();
     m_vertex_cache_ghost.clear();
     m_event_ghost->clear();
-    return 1;
+    return true;
 }
 
 int ReaderAsciiHepMC2::parse_event_information(GenEvent &evt, const char *buf) {
     const char          *cursor             = buf;
-    int                  vertices_count     = 0;
+    size_t               vertices_count     = 0;
     int                  random_states_size = 0;
     int                  weights_size       = 0;
     std::vector<long>    random_states(0);
@@ -364,20 +389,26 @@ int ReaderAsciiHepMC2::parse_event_information(GenEvent &evt, const char *buf) {
     //random states
     if ( !(cursor = strchr(cursor+1, ' ')) ) return -1;
     random_states_size = atoi(cursor);
-    random_states.resize(random_states_size);
-
+    if (random_states_size >= 0 ) {
+        random_states.resize(random_states_size);
+    } else {
+        HEPMC3_DEBUG(0, "ReaderAsciiHepMC2: E: " << evt.event_number() << " (" << vertices_count << "V, "  << random_states_size << "RS)")
+    }
     for ( int i = 0; i < random_states_size; ++i ) {
         if ( !(cursor = strchr(cursor+1, ' ')) ) return -1;
         random_states[i] = atoi(cursor);
     }
 
-    if (random_states.size()) evt.add_attribute("random_states", std::make_shared<VectorLongIntAttribute>(random_states));
+    if (!random_states.empty()) evt.add_attribute("random_states", std::make_shared<VectorLongIntAttribute>(random_states));
 
     // weights
     if ( !(cursor = strchr(cursor+1, ' ')) ) return -1;
     weights_size = atoi(cursor);
-    weights.resize(weights_size);
-
+    if (weights_size >= 0 ) {
+        weights.resize(weights_size);
+    } else {
+        HEPMC3_DEBUG(0, "ReaderAsciiHepMC2: E: " << evt.event_number() << " (" << vertices_count << "V, " << weights_size << "WS)")
+    }
     for ( int i = 0; i < weights_size; ++i ) {
         if ( !(cursor = strchr(cursor+1, ' ')) ) return -1;
         weights[i] = atof(cursor);
@@ -463,16 +494,16 @@ int ReaderAsciiHepMC2::parse_vertex_information(const char *buf) {
 
 
     // Add original vertex barcode to the cache
-    m_vertex_cache.push_back(data);
-    m_vertex_barcodes.push_back(barcode);
+    m_vertex_cache.emplace_back(data);
+    m_vertex_barcodes.emplace_back(barcode);
 
     m_event_ghost->add_vertex(data_ghost);
 
-    if (weights.size()) data_ghost->add_attribute("weights", std::make_shared<VectorDoubleAttribute>(weights));
+    if (!weights.empty()) data_ghost->add_attribute("weights", std::make_shared<VectorDoubleAttribute>(weights));
 
-    m_vertex_cache_ghost.push_back(data_ghost);
+    m_vertex_cache_ghost.emplace_back(data_ghost);
 
-    HEPMC3_DEBUG(10, "ReaderAsciiHepMC2: V: " << -(int)m_vertex_cache.size() << " (old barcode" << barcode << ") " << num_particles_out << " particles)")
+    HEPMC3_DEBUG(10, "ReaderAsciiHepMC2: V: " << -(int)m_vertex_cache.size() << " (old barcode " << barcode << ") " << num_particles_out << " particles)")
 
     return num_particles_out;
 }
@@ -546,7 +577,8 @@ int ReaderAsciiHepMC2::parse_particle_information(const char *buf) {
     if (flowsize)
     {
         std::vector<int> vectorflows;
-        for (auto f: flows) vectorflows.push_back(f.second);
+        vectorflows.reserve(flows.size());
+        for (const auto& f: flows) { vectorflows.emplace_back(f.second); }
         data_ghost->add_attribute("flows", std::make_shared<VectorIntAttribute>(vectorflows));
     }
     // Set prod_vtx link
@@ -558,9 +590,9 @@ int ReaderAsciiHepMC2::parse_particle_information(const char *buf) {
         m_vertex_cache.back()->add_particle_out(data);
     }
 
-    m_particle_cache.push_back(data);
-    m_particle_cache_ghost.push_back(data_ghost);
-    m_end_vertex_barcodes.push_back(end_vtx);
+    m_particle_cache.emplace_back(data);
+    m_particle_cache_ghost.emplace_back(data_ghost);
+    m_end_vertex_barcodes.emplace_back(end_vtx);
 
     HEPMC3_DEBUG(10, "ReaderAsciiHepMC2: P: " << m_particle_cache.size() << " ( pid: " << data->pid() << ") end vertex: " << end_vtx)
 
@@ -572,14 +604,17 @@ bool ReaderAsciiHepMC2::parse_xs_info(GenEvent &evt, const char *buf) {
     std::shared_ptr<GenCrossSection>  xs     = std::make_shared<GenCrossSection>();
 
     if ( !(cursor = strchr(cursor+1, ' ')) ) return false;
-    double xs_val  = atof(cursor);
+    const double xs_val  = atof(cursor);
 
     if ( !(cursor = strchr(cursor+1, ' ')) ) return false;
-    double xs_err = atof(cursor);
-
-    xs->set_cross_section(xs_val, xs_err);
+    const double xs_err = atof(cursor);
+    const size_t all = m_options.count("disable_pad_cross_sections") ? size_t{1} : std::max(evt.weights().size(),size_t{1});
+    const double xs_val_dummy = m_options.count("pad_cross_section_value") ? std::strtod(m_options.at("pad_cross_section_value").c_str(),nullptr) : 0.0;
+    const double xs_err_dummy = m_options.count("pad_cross_section_error") ? std::strtod(m_options.at("pad_cross_section_error").c_str(),nullptr) : 0.0;
+    xs->set_cross_section(std::vector<double>(all,xs_val_dummy), std::vector<double>(all,xs_err_dummy));
+    xs->set_xsec(0,xs_val);
+    xs->set_xsec_err(0,xs_err);
     evt.add_attribute("GenCrossSection", xs);
-
     return true;
 }
 
@@ -696,12 +731,12 @@ bool ReaderAsciiHepMC2::parse_pdf_info(GenEvent &evt, const char *buf) {
     //For compatibility with original HepMC2
     bool pdfids = true;
     if ( !(cursor = strchr(cursor+1, ' ')) ) pdfids = false;
-    if (pdfids) pi->pdf_id[0] = atoi(cursor);
-    else  pi->pdf_id[0] = 0;
+    if (pdfids) {pi->pdf_id[0] = atoi(cursor);}
+    else  {pi->pdf_id[0] = 0;}
 
     if (pdfids) if ( !(cursor = strchr(cursor+1, ' ')) )  pdfids = false;
-    if (pdfids) pi->pdf_id[1] = atoi(cursor);
-    else  pi->pdf_id[1] = 0;
+    if (pdfids) { pi->pdf_id[1] = atoi(cursor);}
+    else  {pi->pdf_id[1] = 0;}
 
     evt.add_attribute("GenPdfInfo", pi);
 
