@@ -1,7 +1,7 @@
 // -*- C++ -*-
 //
 // This file is part of HepMC
-// Copyright (C) 2014-2020 The HepMC collaboration (see AUTHORS for details)
+// Copyright (C) 2014-2023 The HepMC collaboration (see AUTHORS for details)
 //
 ///
 /// @file WriterPlugin.cc
@@ -15,34 +15,33 @@
 #include <intrin.h>
 #include <windows.h>
 #endif
-#if defined(__linux__) || defined(__darwin__)
+#if defined(__linux__) || defined(__darwin__) || defined(__APPLE__) || defined(BSD) || defined(__sun)
 #include <dlfcn.h>
 #endif
-#include <cstring>
 #include <sstream>
-#include "HepMC3/WriterPlugin.h"
+#include <cstring>
 #include "HepMC3/GenEvent.h"
+#include "HepMC3/WriterPlugin.h"
+
 
 
 namespace HepMC3 {
 
 WriterPlugin::WriterPlugin(std::ostream & stream, const std::string &libname, const std::string &newwriter, std::shared_ptr<GenRunInfo> run) {
 #ifdef WIN32
-    dll_handle = nullptr;
     dll_handle = LoadLibrary(libname.c_str());
     if (!dll_handle) { printf("Error  while loading library %s. Error code %i\n", libname.c_str(), GetLastError()); m_writer = nullptr; return;  }
-    typedef Writer* (__stdcall *f_funci)(std::ostream & stream, shared_ptr<GenRunInfo>);
+    typedef Writer* (__stdcall *f_funci)(std::ostream & stream, std::shared_ptr<GenRunInfo>);
     f_funci newWriter = (f_funci)GetProcAddress((HINSTANCE)(dll_handle), newwriter.c_str());
     if (!newWriter) { printf("Error  while loading function %s from  library %s. Error code %i\n", newwriter.c_str(), libname.c_str(), GetLastError()); m_writer = nullptr; return;  }
     m_writer = (Writer*)(newWriter(stream, run));
 #endif
 
-#if defined(__linux__) || defined(__darwin__)
-    dll_handle = nullptr;
+#if defined(__linux__) || defined(__darwin__) || defined(__APPLE__) || defined(BSD) || defined(__sun)
     dll_handle = dlopen(libname.c_str(), RTLD_LAZY | RTLD_GLOBAL);
     if (!dll_handle) { printf("Error  while loading library %s: %s\n", libname.c_str(), dlerror()); m_writer=nullptr; return;  }
-    Writer* (*newWriter)(std::ostream & stream, std::shared_ptr<GenRunInfo>);
-    newWriter = (Writer* (*)(std::ostream & stream, std::shared_ptr<GenRunInfo>))dlsym(dll_handle, newwriter.c_str());
+    using f_funci =  Writer* (*)(std::ostream & stream, std::shared_ptr<GenRunInfo>);
+    auto newWriter = (f_funci)dlsym(dll_handle, newwriter.c_str());
     if (!newWriter) { printf("Error  while loading function %s from  library %s: %s\n", newwriter.c_str(), libname.c_str(), dlerror()); m_writer = nullptr; return;   }
     m_writer = (Writer*)(newWriter(stream, run));
 #endif
@@ -50,21 +49,19 @@ WriterPlugin::WriterPlugin(std::ostream & stream, const std::string &libname, co
 
 WriterPlugin::WriterPlugin(const std::string& filename, const std::string &libname, const std::string &newwriter, std::shared_ptr<GenRunInfo> run) {
 #ifdef WIN32
-    dll_handle = nullptr;
     dll_handle = LoadLibrary(libname.c_str());
     if (!dll_handle) { printf("Error  while loading library %s. Error code %i\n", libname.c_str(), GetLastError()); m_writer = nullptr; return;  }
-    typedef Writer* (__stdcall *f_funci)(const std::string&, shared_ptr<GenRunInfo>);
+    typedef Writer* (__stdcall *f_funci)(const std::string&, std::shared_ptr<GenRunInfo>);
     f_funci newWriter = (f_funci)GetProcAddress((HINSTANCE)(dll_handle), newwriter.c_str());
     if (!newWriter) { printf("Error  while loading function %s from  library %s. Error code %i\n", newwriter.c_str(), libname.c_str(), GetLastError()); m_writer = nullptr; return;  }
     m_writer = (Writer*)(newWriter(filename, run));
 #endif
 
-#if defined(__linux__) || defined(__darwin__)
-    dll_handle = nullptr;
+#if defined(__linux__) || defined(__darwin__) || defined(__APPLE__) || defined(BSD) || defined(__sun)
     dll_handle = dlopen(libname.c_str(), RTLD_LAZY | RTLD_GLOBAL);
     if (!dll_handle) { printf("Error  while loading library %s: %s\n", libname.c_str(), dlerror()); m_writer = nullptr; return;  }
-    Writer* (*newWriter)(const std::string&, std::shared_ptr<GenRunInfo>);
-    newWriter = (Writer* (*)(const std::string&, std::shared_ptr<GenRunInfo>))dlsym(dll_handle, newwriter.c_str());
+    using  f_funci =  Writer* (*)(const std::string&, std::shared_ptr<GenRunInfo>);
+    auto newWriter = (f_funci)dlsym(dll_handle, newwriter.c_str());
     if (!newWriter) { printf("Error  while loading function %s from  library %s: %s\n", newwriter.c_str(), libname.c_str(), dlerror()); m_writer = nullptr; return;   }
     m_writer = (Writer*)(newWriter(filename, run));
 #endif
@@ -78,7 +75,7 @@ WriterPlugin::~WriterPlugin() {
         FreeLibrary((HINSTANCE)dll_handle);
     }
 #endif
-#if defined(__linux__) || defined(__darwin__)
+#if defined(__linux__) || defined(__darwin__) || defined(__APPLE__) || defined(BSD) || defined(__sun)
     if (dll_handle) {
         dlclose(dll_handle);
         dll_handle = nullptr;
